#include "Minesweeper.h"
#include "mymath.h"
#include <iostream>
#include <sstream>

std::string parseInt(int i)
{
	std::ostringstream s;
    s << i;

	return s.str();
}

void print(const field* f)
{
	std::cout << "Bomb: " << (f->bomb ? 1 : 0) << " - Around: " << (f->bombs_around) << " - Visible: " << (f->visible ? 1 : 0)  << "\n";
}

void Minesweeper::increaseCounter(int pos)
{
	if(pos < this->size && pos >= 0)
	{
		this->fields[pos].bombs_around++;
	}
}

Minesweeper::Minesweeper(int width, int height, int bombs, sf::Texture &bomb, sf::Texture &bomb_explosion, sf::Texture &open, sf::Texture &closed, sf::Font &font)
{
	this->fields = new field[width*height];
	this->width = width;
	this->height = height;
	this->size = height*width;
	this->bomb = bomb;
	this->bomb_explosion = bomb_explosion;
	this->open = open;
	this->closed = closed;
	this->text.setFont(font);
	this->text.setCharacterSize(10);
	this->text.setColor(sf::Color::White);
	this->text.setPosition(10, 10);
	this->text.setStyle(sf::Text::Regular);

	this->generateGame(bombs);
}

void Minesweeper::generateGame(int bombs)
{
	this->left_fields = this->width * this->height - bombs;
	field *entry;

	for(int i=this->width*this->height-1;i>=0;i--)
	{
		entry = &this->fields[i];

		entry->bomb = false;
		entry->bombs_around = 0;
		entry->clicked = false;
		entry->visible = false;
		entry->sp.setTexture(this->closed);
		entry->sp.setScale(0.3f, 0.3f);
	}

	int pos;

	// don't allow endless loops
	if(this->size < bombs)
	{
		return;
	}

	// initialize random seed
	srand(time(NULL));

	while(bombs > 0)
	{
		pos = rand(0, size-1);

		if(!this->fields[pos].bomb)
		{
			this->fields[pos].bomb = true;
			
			this->increaseCounter(pos-this->width);
			this->increaseCounter(pos+this->width);

			if(pos%20 -1 == (pos-1)%20)
			{
				this->increaseCounter(pos-1-this->width);
				this->increaseCounter(pos-1);
				this->increaseCounter(pos-1+this->width);
			}

			if(pos%20 + 1 == (pos+1)%20)
			{
				this->increaseCounter(pos+1-this->width);
				this->increaseCounter(pos+1);
				this->increaseCounter(pos+1+this->width);
			}
			
			bombs--;
		}
	}
}

State Minesweeper::activateField(int x, int y)
{
	if(this->width > x && x >= 0 && this->height > y && y >= 0)
	{
		field *entry = &this->fields[y*this->width + x];
		bool oldState = entry->visible;
		entry->visible = true;

		if(entry->visible != oldState)
		{
			this->left_fields--;
		}

		if(entry->bomb)
		{
			entry->clicked = true;
			entry->sp.setTexture(this->bomb_explosion);
			return BOMB;
		}
		else
		{
			entry->sp.setTexture(this->open);

			if(entry->bombs_around == 0 && !oldState)
			{
				this->activateField(x-1, y-1);
				this->activateField(x-1, y);
				this->activateField(x-1, y+1);
				this->activateField(x, y-1);
				this->activateField(x, y+1);
				this->activateField(x+1, y-1);
				this->activateField(x+1, y);
				this->activateField(x+1, y+1);
			}


			if(this->left_fields == 0)
			{
				return END;
			}
		}
	}
	else
	{
		// error?!
	}
}

void Minesweeper::openBombs()
{
	for(int i=this->width*this->height-1;i>=0;i--)
	{
		if(this->fields[i].bomb && !this->fields[i].clicked)
		{
			this->fields[i].visible = true;
			this->fields[i].sp.setTexture(this->bomb);
		}
	}
}

void Minesweeper::printfield(sf::RenderWindow &window, int field_size, int x_start, int y_start)
{
	field *entry;
	sf::String textValue;

	for(int i=0;i<this->width;i++)
	{
		for(int j=0;j<this->height;j++)
		{
			entry = &this->fields[i*this->width + j];

			entry->sp.setPosition(x_start+j*field_size, y_start+i*field_size);

			window.draw(entry->sp);

			if(entry->visible && entry->bombs_around > 0 && !entry->bomb)
			{
				this->text.setString(parseInt(entry->bombs_around));
				this->text.setPosition(j * 40 + 15, i * 40 + 15);
				window.draw(this->text);
			}
		} 
	}
}

Minesweeper::~Minesweeper(void)
{
}

