#include <SFML/Graphics.hpp>
#include <SFML/Audio.hpp>
#include <iostream>
#include "Minesweeper.h"
#include "Menu.h"
#include <stdio.h>  /* defines FILENAME_MAX */
#include <direct.h>
#define GetCurrentDir _getcwd

int main()
{

	char cCurrentPath[FILENAME_MAX];

	if (!GetCurrentDir(cCurrentPath, sizeof(cCurrentPath)))
	{
		return errno;
	}

	cCurrentPath[sizeof(cCurrentPath) - 1] = '\0'; /* not really required */

	std::string path = cCurrentPath;

	std::cout << path << std::endl;

	// load explosion sound
	sf::SoundBuffer buffer;
    buffer.loadFromFile("sound/bomb.ogg");
    sf::Sound sound_explosion = sf::Sound(buffer);

	sf::SoundBuffer buffer2;
    buffer2.loadFromFile("sound/win.wav");
    sf::Sound sound_win = sf::Sound(buffer2);
	
    sf::Texture t_bomb;
	t_bomb.loadFromFile("gfx/bomb.png");
	t_bomb.setSmooth(true);

	sf::Texture t_bomb_explosion;
	t_bomb_explosion.loadFromFile("gfx/bomb_explosion.png");
	t_bomb_explosion.setSmooth(true);

	sf::Texture t_open;
	t_open.loadFromFile("gfx/open.png");
	t_open.setSmooth(true);

	sf::Texture t_closed;
	t_closed.loadFromFile("gfx/closed.jpg");
	t_closed.setSmooth(true);

	sf::Texture t_hover;
	t_hover.loadFromFile("gfx/hover.png");
	t_hover.setSmooth(true);

	sf::Texture t_explosion;
	t_explosion.loadFromFile("gfx/explosion.png");
	t_explosion.setSmooth(true);

	sf::Sprite s_hover;
	s_hover.setTexture(t_hover);

	sf::Sprite s_explosion;
	s_explosion.setTexture(t_explosion);

	sf::RenderWindow window(sf::VideoMode(800, 800), "Minesweeper");
	
	sf::Font font;
	if (!font.loadFromFile("C:\\Windows\\Fonts\\Calibri.ttf"))
	{
		// error...
	}

	
	sf::Text text1;
	text1.setFont(font);
	text1.setCharacterSize(10);
	text1.setColor(sf::Color::White);
	text1.setPosition(10, 10);
	text1.setStyle(sf::Text::Regular);

	std::cout << system("cd .");

	sf::Vector2i localPosition;
	sf::Vector2i oldLocalPosition;
	std::string text;
	sf::Event event;
	State st;

	Minesweeper ms(20, 20, 25, t_bomb, t_bomb_explosion, t_open, t_closed, font);

	float x = 0;
	float y = 0;
	bool run = false;
	sf::Text finishText;
	finishText.setFont(font);
	finishText.setCharacterSize(36);
	finishText.setColor(sf::Color::Red);
	finishText.setPosition(250, 250);
	finishText.setStyle(sf::Text::Regular);

	Menu menu(font);
	menu.addPoint("Spiel starten (10)");
	menu.addPoint("Spiel starten (20)");
	menu.addPoint("Spiel starten (40)");
	menu.addPoint("Spiel starten (75)");
	menu.addPoint("Highscore");
	menu.addPoint("Spiel beenden");

	bool key = false;
	bool mouse = false;
	bool performMenu = false;
	bool runExplosion = false;
	int explosionOffset = 0;
	int explosionOffsetX = 0;
	int explosionOffsetY = 0;
	const int explosionSize = 64;
	
	while (window.isOpen())
    {
		while (window.pollEvent(event))
		{
			if (event.type == sf::Event::Closed)
			{
				window.close();
			}

			localPosition = sf::Mouse::getPosition(window);
			text = "Mausposition: "+std::to_string(localPosition.x);
			text += ":"+std::to_string(localPosition.y);

			text1.setString(text);
		}
		
		if(run)
		{
			x = (localPosition.x-localPosition.x%40)/40;
			y = (localPosition.y-localPosition.y%40)/40;

			window.clear();

			if (sf::Mouse::isButtonPressed(sf::Mouse::Left))
			{
				if(!mouse)
				{
					mouse = true;
					st = ms.activateField(x, y);

					switch(st)
					{
						case BOMB:
							finishText.setString("Bombe erwischt, sry!");
							ms.openBombs();
							run = false;
							runExplosion = true;
							s_explosion.setPosition(x*40 - 15, y*40 - 15);
							break;
						case OK:

							break;
						case END:
							finishText.setString("Alle Bomben entdeckt!");
							ms.openBombs();
							sound_win.setPlayingOffset(sf::seconds(0));
							sound_win.play();
							run = false;
							break;
					}
				}
			}
			else
			{
				mouse = false;
			}
		}
		else
		{
			if(sf::Keyboard::isKeyPressed(sf::Keyboard::Up))
			{
				if(!key)
				{
					key = true;
					menu.setPreviousActive();
				}
			}
			else if(sf::Keyboard::isKeyPressed(sf::Keyboard::Down))
			{
				if(!key)
				{
					key = true;
					menu.setNextActive();
				}
			}
			else if(sf::Keyboard::isKeyPressed(sf::Keyboard::Return) || performMenu)
			{
				if(!key || performMenu)
				{
					key = true;
					performMenu = false;

					switch(menu.getActiveIndex())
					{
						case 0:
							run = true;
							finishText.setString("");
							ms.generateGame(10);
							runExplosion = false;
							break;
						case 1:
							run = true;
							finishText.setString("");
							ms.generateGame(20);
							runExplosion = false;
							break;
						case 2:
							run = true;
							finishText.setString("");
							ms.generateGame(40);
							runExplosion = false;
							break;
						case 3:
							run = true;
							finishText.setString("");
							ms.generateGame(75);
							runExplosion = false;
							break;

						// highscore
						case 4:
							finishText.setString("no highscore yet!");
							break;

						// quit
						case 5:
							return 0;
							break;
					}
				}
			}
			else
			{
				if(oldLocalPosition.x != localPosition.x || oldLocalPosition.y != localPosition.y)
				{
					menu.selectMenuPoint(localPosition);
				}

				if(sf::Mouse::isButtonPressed(sf::Mouse::Left))
				{
					if(menu.selectMenuPoint(localPosition))
					{
						performMenu = true;
					}
				}

				key = false;
			}
		}

		ms.printfield(window, 40, 0, 0);

		//window.draw(text1);
		window.draw(finishText);

		if(run)
		{
			s_hover.setPosition(x*40, y*40);
			window.draw(s_hover);
		}
		else
		{
			menu.drawMenu(window);
		}

		if(runExplosion)
		{
			if(explosionOffset == 0)
			{
				sound_explosion.setPlayingOffset(sf::seconds(0));
				sound_explosion.play();
			}

			explosionOffset++;

			if(explosionOffset % 15 == 0)
			{
				explosionOffsetX++;

				if(explosionOffsetX == 5)
				{
					explosionOffsetX = 0;
					explosionOffsetY++;
				}
			}

			s_explosion.setTextureRect(sf::IntRect(explosionOffsetX*explosionSize, explosionOffsetY*explosionSize, explosionSize, explosionSize));

			window.draw(s_explosion);

			if(explosionOffset >= 375)
			{
				sound_explosion.stop();
				explosionOffset = 0;
				explosionOffsetX = 0;
				explosionOffsetY = 0;
				runExplosion = false;
			}
		}

		oldLocalPosition = localPosition;

		window.display();
    }

	sound_explosion.stop();
	sound_win.stop();

    return 0;
}